function s = clock2str
%  return the clock time as a string in a nice format
%  Written by M. Overton (overton@cs.nyu.edu), last revised March 2003
c = clock;
s = [num2str(c(1)) '-' num2str(c(2)) '-' num2str(c(3)) '-' ...
     num2str(c(4)) '-' num2str(c(5))];    % can't use : on PC

