% README.m
% Gradient Sampling Minimization Files

% sampledemo.m       execute this first, by typing: sampledemo
% samplefun.m        a sample function to be minimized
% gradsamp.m*        the main program, see comments for parameter details

% You must have either MOSEK or the Matlab Optimization Toolbox installed.
% MOSEK is recommended: www.mosek.com.
% If both are installed, which is called will depend on the Matlab path.
% To ensure MOSEK is called, use "addpath ..." to add it to the front of
% the path.

% Supporting files:

% autoreduce.m        
% clock2str.m         
% getbundle.m         
% gradsampfixed.m     
% fcomparedefault.m   
% findbest.m          
% isinteger.m         
% lpsubprob.m   (not normally called)
% ndlinesch.m         
% qpsubprob.m         
% stepmax.m
